#!/bin/bash

. /etc/init.d/functions
. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

ssh_port=22
TCP_REJECT="REJECT --reject-with tcp-reset"
readonly MAINTAINANCE_REQUIRED="${MAINTAINANCE_REQUIRED:-0}"

create_and_flush_chain() {
	chain="$1"
	table="${2:-filter}"
	iptables -t $table -N $chain 2>/dev/null || true
	iptables -t $table -F $chain
}

add_skip_check_rule() {
	if [ -s $HOOKDIR/skip_check.list ]; then
		echo  "- Добавляется правило DROP для белого списка абонентов"
		iptables -A FORWARD -m set --set skip_check src -j DROP
	fi
}

custom_bindings_exists() {
	[ -s $HOOKDIR/custom_bindings ] && return 0
	[ -s $HOOKDIR/custom_bindings_dns ] && return 0
	[ -s $HOOKDIR/custom_bindings_https ] && return 0
	return 1
}

add_custom_rules_http() {
	if [ -s $HOOKDIR/custom_bindings ]; then
		local FORBIDDEN_TARGET="$1"
		log "Добавляются дополнительные правила фильтрации http"
		while read blacklist_id ipset_id redirect_url comment; do
			if [ "$ipset_id" = '-1' ] && [ "$blacklist_id" = '0' ]; then
				log "Удаляем дефолтное фильтрующее правило из-за записи в $HOOKDIR/custom_bindings: $blacklist_id $ipset_id $redirect_url $comment"
				chroot $CHROOTDIR /sbin/iptables -A 'http' -p tcp -m reductor --id $blacklist_id --comment "$comment" -j $FORBIDDEN_TARGET $redirect_url
				iptables -D http 1
				continue
			fi
			chroot $CHROOTDIR /sbin/iptables -A 'http' -p tcp -m set --match-set custom_$ipset_id src -m reductor --id $blacklist_id --comment "$comment" -j $FORBIDDEN_TARGET $redirect_url
		done < $HOOKDIR/custom_bindings
	fi
}

add_custom_rules_dns() {
	local bindings="$HOOKDIR/custom_bindings_dns"
	local list_id ipset_id redirect_ip exact_mode blackwhite comment
	if [ ! -s "$bindings" ]; then
		return 0
	fi
	while read list_id ipset_id redirect_ip exact_mode blackwhite comment; do
		if [ "$exact_mode" = 'exact' ]; then
			exact_mode=' --exact'
		elif [ "$exact_mode" = 'with_subdomains' ]; then
			exact_mode=''
		else
			log "WARNING: не указан exact_mode для $list_id $ipset_id $redirect_ip $exact_mode $comment"
			continue
		fi
		if [ "$blackwhite" != 'black' ] && [ "$blackwhite" != 'white' ]; then
			log "WARNING: неизвестно, является ли список $list_id $ipset_id $redirect_ip $exact_mode $comment чёрным или белым"
			continue
		fi
		# shellcheck disable=SC2154
		if [ "$blackwhite" = 'black' ] && ! echo "$redirect_ip" | egrep -q "$ip_regex"; then
			log "WARNING: IP редиректа $redirect_ip не является IP адресом ($comment)"
			continue
		fi
		if [ "$ipset_id" = '-' ]; then
			ipset_id=''
		else
			ipset_id="-m set --match-set custom_$ipset_id src"
		fi
		if [ "$blackwhite" = 'black' ]; then
			iptables -A dns -p udp $ipset_id -m dnsmatch --id $list_id $exact_mode --comment "$comment" -j DNSREPLY --dnsreply-ip "$redirect_ip"
		else
			iptables -I dns -p udp $ipset_id -m dnsmatch --id $list_id $exact_mode --comment "$comment" -j RETURN
		fi
	done < "$bindings"
}

add_custom_rules_https() {
	if [ -s $HOOKDIR/custom_bindings_https ]; then
		local TARGET="$TCP_REJECT"
		log "Добавляются дополнительные правила фильтрации https"
		while read src dst _; do
			iptables -A https_dst -p tcp -m set --match-set custom_$src src -m set --match-set custom_$dst dst -j $TARGET
			iptables -A https_src -p tcp -m set --match-set custom_$src src -m set --match-set custom_$dst dst -j $TARGET
		done < $HOOKDIR/custom_bindings_https
	fi
}

admin_notify() {
	local is_readonly
	local page="http://deny.carbonsoft.ru/whereismymoney"
	is_readonly="$(python -m json.tool /var/reg/CRB-Reductor.key | grep '"cf_regreadonly":' | egrep -o [0-9])" || true
	# shellcheck disable=SC2154
	if [ "${misc['admin_notify']:-0}" = '1' ] && [ "$is_readonly" = '1' ]; then
		log "Добавляем напоминание для администраторов о скорой блокировке"
		iptables -I http ! -d $CARBONSOFT_BLACKHOLE -p tcp --tcp-flags PSH PSH -m set --set reductor_admins src -m limit --limit 5/hour -j FORBIDDEN --forbidden-url $page
	fi
}

http() {
	log "Настраиваем фильтрацию http"
	create_and_flush_chain "http"
	local TARGET="$TCP_REJECT"
	local FORBIDDEN_TARGET="FORBIDDEN --forbidden-url "
	if [ "${filter['forbidden']}" = '1' ]; then
		local url="${filter['forbidden_url']:-http://helpdesk.carbonsoft.ru/deny}"
		TARGET="FORBIDDEN"
		[ "${filter['save_domain']:-0}" == '1' ] && TARGET="$TARGET --save-domain"
		[ "${filter['forbidden_log']:-0}" == '1' ] && TARGET="$TARGET --log"
		FORBIDDEN_TARGET="$TARGET --forbidden-url" #for use in custom redirect
		TARGET="$FORBIDDEN_TARGET $url"
	fi
	local traffic_to_filter="-p tcp -m set --match-set port_http dst"
	if [ -s $LISTDIR/provider/our.white_http_iplist ]; then
		log "Добавляем белый список ip (http)"
		iptables -A http -m set --match-set white_http_reductor dst -j DROP
	fi
	chroot $CHROOTDIR /sbin/iptables -A 'http' -p tcp -m reductor --id 0 --comment 'rkn' -j $TARGET
	admin_notify
	add_custom_rules_http "$FORBIDDEN_TARGET"
	iptables -A FORWARD $traffic_to_filter -j http
}

sni() {
	local nginx_ip
	[ "${filter['sni']:-0}" = '1' ] || return 0
	log "Настраиваем фильтрацию HTTPS по SNI"
	insert_module xt_snimatch
	if ! $BINDIR/dnsenabled; then
		iptables -A https_dst -m length --length 61: -p tcp -m snimatch --id 0 --comment 'sni https?' -j $TCP_REJECT
		return
	fi
	if nginx_ip="$($BINDIR/dnsenabled --nginx-ip)"; then
		iptables -A https_dst -d $nginx_ip -j DROP
	fi
	if $BINDIR/dnsenabled --proxy-may-be-used; then
		#TODO: proxy can be installed on the same machine
		iptables -A https_dst -d ${filter['dns_proxy_ip']} -j DROP
	fi
	iptables -A https_dst -m length --length 61: -p tcp -m snimatch --id 0 --comment 'sni https?' -j $TCP_REJECT
}

ip_https() {
	for direct in dst src; do
		for ipset in ip_https ip_https_plus; do
			if [ "${filter['https_by_ip']}" = '0' ] && [ "$ipset" = 'ip_https_plus' ]; then
				log "Блокируем HTTPS по IP только в крайнем случае."
				continue
			fi
			iptables -A https_$direct -p tcp -m set --match-set $ipset $direct -j $TCP_REJECT
		done
	done
}

ip_https_whitelist() {
	if [ -s $LISTDIR/provider/our.white_https_iplist ]; then
		log "Добавляем белый список ip (https)"
		iptables -A https_dst -m set --match-set white_https_reductor dst -j DROP
		iptables -A https_src -m set --match-set white_https_reductor src -j DROP
	fi
}

https() {
	log "Настраиваем фильтрацию https"
	create_and_flush_chain "https_src"
	create_and_flush_chain "https_dst"
	ip_https_whitelist
	# sni после ip_https т.к. незачем напрягать sni, если это всё равно отреджектится по ip
	ip_https
	sni
	add_custom_rules_https
	iptables -A FORWARD -p tcp -m set --match-set port_https dst -j https_dst
	iptables -A FORWARD -p tcp -m set --match-set port_https src -j https_src
}

ip_port() {
	if [ "${filter['cardsharing']:-}" = '1' ]; then
		log "Настраиваем фильтрацию кардшаринга и прочего"
		create_and_flush_chain "other_traffic"
		for side in src dst; do
			iptables -A other_traffic -p tcp -m set --match-set ip_tcp_udp $side -m set --match-set port_tcp $side -j $TCP_REJECT
			iptables -A other_traffic -p udp -m set --match-set ip_tcp_udp $side -m set --match-set port_udp $side -j REJECT --reject-with icmp-port-unreachable
		done
		iptables -A FORWARD -j other_traffic
	fi
}

filter_policy() {
	iptables -P INPUT ACCEPT
	iptables -P FORWARD DROP
	iptables -P OUTPUT ACCEPT
}

dns_post_revisor_hook() {
	return 0
}

dns() {
	$BINDIR/dnsenabled || return 0
	local IP
	local icase=""
	IP="$($BINDIR/dnsenabled --nginx-ip)"
	log "Настраиваем фильтрацию DNS"
	if [ "$IP" != "${IP%%:*}" ]; then
		$SETCOLOR_FAILURE
		log "WARNING: обрезаем вероятно порт, указанный рядом с IP, DNS ничего не знает о портах"
		$SETCOLOR_NORMAL
		IP="${IP%%:*}"
	fi
	insert_module xt_dnsmatch
	insert_module ipt_DNSREPLY
	create_and_flush_chain "dns"

	if $BINDIR/dnsenabled --revisor; then
		log "Добавляем правила icase-спуфинга для ревизора"
		create_and_flush_chain "dns_revisor"
		iptables -I dns -s "${filter['revisor_ip']}" -j dns_revisor
		dns_post_revisor_hook subchain
	else
		icase=" --icase"
	fi

	local MASK_MATCH="-p udp -m dnsmatch --id 0 --comment domain_mask $icase"
	local MASK_TARGET="DNSREPLY --dnsreply-ip $IP"
	iptables -A dns $MASK_MATCH -j $MASK_TARGET
	if $BINDIR/dnsenabled --revisor; then
		iptables -A dns_revisor $MASK_MATCH --icase -j $MASK_TARGET
	fi

	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		local EXACT_MATCH="-p udp -m dnsmatch --id 1000 --exact --comment domain_exact $icase"
		local EXACT_TARGET="DNSREPLY --dnsreply-ip $IP"
		iptables -A dns $EXACT_MATCH -j $EXACT_TARGET
		if $BINDIR/dnsenabled --revisor; then
			iptables -A dns_revisor $EXACT_MATCH --icase -j $EXACT_TARGET
		fi
	fi

	if $BINDIR/dnsenabled --proxy-https; then
		local PROXY_HTTPS_MATCH="! -s ${filter['dns_proxy_ip']} -p udp -m dnsmatch --id 2000 --comment domain_proxy $icase"
		local PROXY_HTTPS_TARGET="DNSREPLY --dnsreply-ip ${filter['dns_proxy_ip']}"
		iptables -A dns $PROXY_HTTPS_MATCH -j $PROXY_HTTPS_TARGET
		if $BINDIR/dnsenabled --revisor; then
			iptables -A dns_revisor $PROXY_HTTPS_MATCH --icase -j $PROXY_HTTPS_TARGET
		fi
	fi

	if $BINDIR/dnsenabled --proxy-hsts; then
		local PROXY_HSTS_MATCH="! -s ${filter['dns_proxy_ip']} -p udp -m dnsmatch --id 3000 --comment domain_proxy $icase"
		local PROXY_HSTS_TARGET="DNSREPLY --dnsreply-ip ${filter['dns_proxy_ip']}"
		iptables -A dns $PROXY_HSTS_MATCH -j $PROXY_HSTS_TARGET
		if $BINDIR/dnsenabled --revisor; then
			iptables -A dns_revisor $PROXY_HSTS_MATCH --icase -j $PROXY_HSTS_TARGET
		fi
	fi

	iptables -A FORWARD -p udp --dport 53 -j dns
	if $BINDIR/dnsenabled --log-enabled; then
		$SETCOLOR_FAILURE
		log "WARNING: включаем DNS логирование, отключите его после завершения дебага"
		$SETCOLOR_NORMAL
		echo "${filter['dns_log_level']}" > /proc/net/ipt_dnsmatch/trace_ctl
	fi
	add_custom_rules_dns
}

revisor_hook() {
	return 0
}

add_forward_for_ip() {
	local ext_ip="$1"
	local ext_dev="$2"
	local ip="$3"
	log "Выпускаем $ip в интернет через себя под IP $ext_ip через $ext_dev"
	iptables -A FORWARD -s "$ip" -j ACCEPT
	iptables -A FORWARD -d "$ip" -j ACCEPT
	iptables -t nat -A POSTROUTING -o "$ext_dev" -s "$ip" -j SNAT --to-source "$ext_ip"
}

revisor() {
	local ext_ip
	local ext_dev
	[ "${filter['revisor_ip']// /}" != "" ] || return 0
	[ "${filter['revisor_route_and_nat']}" = '1' ] || return 0
	if ! ext_ip="$($BINDIR/dnsenabled --ext-ip)"; then
		echo "Cant determine default route"
		return 0
	fi
	if ! ext_dev="$($BINDIR/dnsenabled --ext-dev)"; then
		echo "Cant determine default route dev"
		return 0
	fi
	if [ -z "$ext_ip" ] || [ -z "$ext_dev" ]; then
		echo "WARNING: Не удалось определить внешний ip и устройство для ревизора"
		return 0
	fi
	add_forward_for_ip "$ext_ip" "$ext_dev" "${filter['revisor_ip']}"
	revisor_hook "$ext_ip" "$ext_dev"
}

filter() {
	local WEB_INTERFACE_IP="${admin['web_ip']:-0.0.0.0/0}"
	local WEB_INTERFACE_PORT="${admin['web_port']:-8080}"
	filter_policy
	iptables -F
	if [ "$MAINTAINANCE_REQUIRED" = 1 ]; then
		$BINDIR/maintain.sh lock
	fi
	iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
	iptables -A INPUT -p icmp -j ACCEPT
	iptables -A INPUT -i lo -j ACCEPT
	if [ -n "${admin['ip']:-}" ]; then
		if [ "${filter['forbidden_url_local']:-0}" == '1' ]; then
			iptables -A INPUT -m state --state NEW -p tcp --dport 80 -j ACCEPT
			if $BINDIR/dnsenabled; then
				iptables -A INPUT -m state --state NEW -p tcp --dport 443 -j ACCEPT
			fi
		fi
		if [ "$WEB_INTERFACE_PORT" = '80' ] && [ ! -f $USERDIR/hooks/yes_i_want_web_on_80_port ]; then
			WEB_INTERFACE_PORT="8080"
		fi
		iptables -A INPUT -m state --state NEW -p tcp -d "${WEB_INTERFACE_IP}" --dport "${WEB_INTERFACE_PORT:-8080}" -m set --match-set reductor_admins src -j ACCEPT
	fi
	iptables -A INPUT -m state --state NEW -m tcp -p tcp --dport $ssh_port -j ACCEPT
	iptables -A INPUT -j REJECT --reject-with icmp-host-prohibited

	add_skip_check_rule
	ip_block
	https
	http
	dns
	ip_port
	revisor
}

ip_block() {
	log "Настраиваем фильтрацию IP адресов"
	create_and_flush_chain "ip_block"
	iptables -A ip_block -p tcp -j $TCP_REJECT
	iptables -A ip_block -p udp -j REJECT --reject-with icmp-port-unreachable
	iptables -A ip_block -j REJECT --reject-with icmp-proto-unreachable
	iptables -A FORWARD -m set --match-set ip_block dst,dst -j ip_block
}

mangle() {
	iptables -t mangle -F
	for chain in PREROUTING INPUT FORWARD OUTPUT POSTROUTING; do
		iptables -t mangle -P $chain ACCEPT
	done
}

nat() {
	iptables -t nat -F
	if [ "${filter['router']}" != '1' ]; then
		return 0
	fi
	log "Включается маршрутизация"
	iptables -A FORWARD -j ACCEPT
	{
		iptables -t nat -D POSTROUTING -j MASQUERADE || true
		iptables -t nat -D POSTROUTING -j MASQUERADE || true
	} &>/dev/null
	iptables -t nat -I POSTROUTING -j MASQUERADE
}

raw() {
	local EXT_IP
	iptables -t raw -F PREROUTING
	for chain in mirror_traffic ip_port; do
		create_and_flush_chain "$chain" "raw"
	done
	for link in $(brctl show | egrep -o eth[0-9\.]+$); do
		iptables -t raw -I PREROUTING -i $link -j mirror_traffic
	done
	if [ "${filter['notrack']:-0}" == '1' ]; then
		if EXT_IP="$($BINDIR/dnsenabled --ext-ip)"; then
			if [ -n "$EXT_IP" ]; then
				iptables -t raw -I mirror_traffic -s $EXT_IP -j DROP || true
				iptables -t raw -I mirror_traffic -d $EXT_IP -j DROP || true
			fi
			iptables -t raw -A mirror_traffic -j NOTRACK
		fi
	fi
	iptables -t raw -A mirror_traffic -m ttl --ttl-eq 1 -j DROP
	if [ "${misc['optimize_routing']}" = '1' ]; then
		iptables -t raw -A mirror_traffic -p tcp -m set --match-set port_http dst -j ACCEPT
		# iptables -t raw -A mirror_traffic -p tcp --dport 443 --tcp-flags SYN SYN -j ACCEPT
		iptables -t raw -A mirror_traffic -p tcp -m set --match-set port_https dst -j ACCEPT
		if [ "${filter['dns']:-0}" = '1' ]; then
			iptables -t raw -A mirror_traffic -p udp --dport 53 -j ACCEPT
		fi
		iptables -t raw -A mirror_traffic -m set --match-set ip_tcp_udp dst -j ACCEPT
		iptables -t raw -A mirror_traffic -m set --match-set ip_block dst -j ACCEPT
		iptables -t raw -A mirror_traffic -j DROP
	fi

}

ipset_recreate() {
	local name="$1"
	ipset save | grep -q "^create $*" && return 0
	if ipset save | grep -q "^create $name"; then
		ipset flush "$name"
		ipset destroy "$name" || true
	fi
	log "Пересоздаём $name"
	ipset create "$@"
}

reductor_admins() {
	# ok, no one reads a help text :(
	admin['ip']="${admin['ip']//[,;]/ }"
	ipset flush reductor_admins
	for ip in ${admin['ip']:-}; do
		ipset add reductor_admins $ip
	done
}

__ipset_config() {
	echo reductor_admins hash:net family inet hashsize 256 maxelem 512
	echo ip_https hash:net family inet hashsize 8192 maxelem 65536
	echo ip_https_plus hash:net family inet hashsize 8192 maxelem 65536
	if [ "${filter['ipv6']:-0}" = '1' ]; then
		echo ip6_https hash:net family inet6 hashsize 4096 maxelem 65536
		echo ip6_https_plus hash:net family inet6 hashsize 4096 maxelem 65536
	fi
	for ipset in ip_block white_https_reductor white_http_reductor skip_check; do
		echo "$ipset" hash:net family inet hashsize 1024 maxelem 65536
	done
	echo port_http bitmap:port range 0-65535
	echo port_https bitmap:port range 0-65535
	echo port_tcp bitmap:port range 0-65535
	echo port_udp bitmap:port range 0-65535
	echo ip_tcp_udp hash:ip family inet hashsize 1024 maxelem 2048

	custom_bindings_exists || return 0
	mkdir -p $HOOKDIR/custom_ipsets/
	# shellcheck disable=SC2044
	for ipset in $(find $HOOKDIR/custom_ipsets/ -type f); do
		echo custom_${ipset##*/} hash:net family inet hashsize 1024 maxelem 65536
	done
}

ipset_state() {
	for ipset in $(ipset_config | awk '{print $1}'); do
		ipset save | grep "^create $ipset" | sed 's/create //; s/ $//'
	done | LANG='' sort -u
}

ipset_config() {
	__ipset_config | LANG='' sort -u
}

ipset_create() {
	ipset_config | while read params; do
	ipset_recreate $params
done
}

skip_check() {
	if [ -s $HOOKDIR/skip_check.list ]; then
		log "Заполняем список пропуска проверок"
		ipset flush skip_check
		while read ip _; do
			if [ "${ip:0:1}" != '#' ]; then
				ipset add skip_check $ip
			fi
		done < $HOOKDIR/skip_check.list
	fi
}

custom_bindings() {
	if custom_bindings_exists; then
		mkdir -p $HOOKDIR/custom_ipsets/
		# shellcheck disable=SC2044
		for ipset in $(find $HOOKDIR/custom_ipsets/ -type f); do
			ipset flush custom_${ipset##*/}
			while read ip _; do
				ipset add custom_${ipset##*/} $ip || true
			done < $ipset
		done
	fi
}

ipset_prepare() {
	ipset_create
	skip_check
	custom_bindings
	reductor_admins
}

ipv6() {
	log "Настраиваем фильтрацию IPv6"
	local traffic_to_filter="-p tcp -m set --match-set port_http dst"
	local TARGET="$TCP_REJECT"
	if [ "${filter['forbidden']}" = '1' ]; then
		local url="${filter['forbidden_url']:-http://helpdesk.carbonsoft.ru/deny}"
		TARGET="FORBIDDEN --forbidden-url $url"
		if [ "${filter['save_domain']:-0}" == '1' ]; then
			TARGET="$TARGET --save-domain"
		fi
		if [ "${filter['forbidden_log']:-0}" == '1' ]; then
			TARGET="$TARGET --log"
		fi
	fi

	echo 0 > /proc/sys/net/ipv6/conf/all/disable_ipv6
	echo 0 > /proc/sys/net/ipv6/conf/default/disable_ipv6
	echo 1 > /proc/sys/net/ipv6/conf/all/forwarding
	modprobe ip6_tables
	modprobe ip6table_mangle
	modprobe ip6table_filter
	insert_module ip6t_FORBIDDEN
	if $BINDIR/dnsenabled; then
		insert_module ip6t_DNSREPLY
	fi
	ebtables -t broute -A BROUTING -p IPv6 -j redirect  --redirect-target DROP
	ip6tables -P FORWARD DROP
	ip6tables -F
	chroot $CHROOTDIR /sbin/ip6tables -A FORWARD $traffic_to_filter -m reductor --id 0 --comment 'rkn' -j $TARGET
	if $BINDIR/dnsenabled; then
		local IP="${filter['dns_ip']:-$CARBONSOFT_BLACKHOLE}"
		chroot $CHROOTDIR /sbin/ip6tables -A FORWARD -p udp --dport 53 -m dnsmatch --id 0 --comment 'rkn' -j DNSREPLY --dnsreply-ip $IP
	fi
	ip6tables -N https &>/dev/null || true
	ip6tables -F https
	for direct in src dst; do
		ip6tables -A FORWARD -p tcp -m set --match-set port_https $direct -j https
		for ipset in ip6_https ip6_https_plus; do
			ip6tables -A https -p tcp -m set --match-set $ipset $direct -j $TCP_REJECT
		done
	done
}

ipset_need_recreate() {
	# 0 - reload, 1 - skip
	ipset_config > $TMPDIR/ipset_config
	ipset_state > $TMPDIR/ipset_state
	! cmp $TMPDIR/ipset_config $TMPDIR/ipset_state
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		# shellcheck disable=SC1090
		. "$hook" __include
	fi
}

modules_checksum() {
	find $CHROOTDIR/{lib/modules/,lib64/xtables/} -type f -name "*.[sk]o" -exec md5sum  {} \; | md5sum
}

modules_need_reload() {
	# 0 - reload, 1 - skip
	local checksum_file="/var/lib/reductor/modules_checksum"
	local new_checksum
	new_checksum="$(modules_checksum)"
	mkdir -p /var/lib/reductor/
	touch $checksum_file
	[ ! -s $checksum_file ] && echo "$new_checksum" > $checksum_file && return 0
	read old_checksum < $checksum_file
	echo "$new_checksum" > $checksum_file
	[ "$new_checksum" != "$old_checksum" ]
}

config_hook() {
	if [ "${filter['dns_ip']}" = '127.0.0.1' ]; then
		$SETCOLOR_FAILURE
		echo "  WARNING: Поскольку не настроена DNS-фильтрация, конфигурируем её автоматически,"
		echo "           потому что очень беспокоимся о штрафах со стороны Роскомнадзора, а эта функция"
		echo "           помогает решить имеющуюся проблему с фильтрацией ресурсов с частом меняющимися IP адресами"
		echo "           'Ненастроенность' определяется тем, что одновременно:"
		echo "           1) Отключена DNS-фильтрация"
		echo "           2) IP для редиректа равен 127.0.0.1"
		echo "           Для настройки запустите menu - настройка алгоритма фильтрации"
		$SETCOLOR_NORMAL
		filter['dns']='1'
		filter['dns_ip']="$CARBONSOFT_BLACKHOLE"
	fi
}

start() {
	log "Конфигурация файрвола"
	config_hook
	insert_module xt_reductor
	[ "${filter['forbidden']}" = '1' ] && insert_module ipt_FORBIDDEN
	ipset_prepare
	raw
	mangle
	nat
	filter
	ebtables-restore < $MAINDIR/contrib/etc/sysconfig/ebtables
	[ "${filter['ipv6']:-0}" = '1' ] && ipv6
	[ -x $HOOKDIR/firewall.sh ] && $HOOKDIR/firewall.sh start || true
	log "Конфигурация файрвола завершена"
}

use_hook
case "${1//[_-]/}" in
	start )
		trap __exit EXIT
		set -eu
		"${1//[_-]/}"
		;;
	ipsetconfig )
		ipset_config
		;;
	ipsetstate )
		ipset_state
		;;
	ipsetneedrecreate )
		ipset_need_recreate
		;;
	modulesneedreload )
		modules_need_reload
		;;
	* )
		echo "$1 not supported, avaible options:"
		grep -A 1000 "case" $0 | grep '[a-z] )' | egrep -o [a-z]+ | sed 's/^/--/g'
		exit 1
		;;
esac
