#!/usr/bin/python
# coding:utf-8
u"""Множество функций для преобразования URL """

import os
import re
from urllib import quote, unquote

__author__ = 'oleg'

PROTO = os.environ.get('proto', 'http')
SCHEMA = PROTO + '://'
SCHEMA_LEN = len(SCHEMA)
MAX_URL_LEN = 600
NOT_IDNA_HOST = re.compile('^[a-zA-Z:\\._0-9-]+$')
QUOTE_SAFE_VARIANTS = (
    '/',
    '/#&(),*+:;=?@~',
    '/#&(),*+:;=?@~!',
    '/#&(),*+:;=?@~![]'
)


def idna(url_set_in):
    u"""Кодирует кириллические домены """
    out = url_set_in.copy()
    for url in url_set_in:
        host = pick_host(url)
        if NOT_IDNA_HOST.match(host):
            continue
        out.add(url.replace(host, host.encode('idna'), 1))
        out.remove(url)
    return out


def lower(url_set_in):
    u"""Дублирует домены с uppercase символами в lowercase """
    out = url_set_in.copy()
    for url in (url for url in url_set_in if not url.islower()):
        host = pick_host(url)
        out.add(re.sub("(?i)" + host, host.lower(), url))
    return out


def www(url_set_in):
    u"""Убирает www если надо """
    out = url_set_in.copy()
    for url in (url for url in url_set_in if url[SCHEMA_LEN:11] == u'www.'):
        out.add(url.replace(u'www.', u'', 1))
        if pick_host(url).count(u'.') < 3:
            out.remove(url)
    return out


def fqdn(url_set_in):
    u"""Дублирует домены с точкой и без точки в конце """
    out = set()
    for url in url_set_in:
        host = pick_host(url)
        if host.endswith('.'):  # is fqdn host
            out.add(url.replace(host, host.rstrip('.')))
        else:  # is norm host
            out.add(url)
    return out


def cp1251(url_set_in):
    u"""Дублирует url в кодировке cp1251 """
    out = url_set_in.copy()
    for url in url_set_in:
        out.add(url.encode('cp1251'))
    return out


def domain_change(url_set_in):
    u"""Дублирует пары доменов, чтобы не добавлять руками 2 раза 1 GET """
    replaces = {
        "vk.com": "vkontakte.ru",
        "vkontakte.ru": "vk.com"
    }
    out = url_set_in.copy()
    for url in url_set_in:
        host = pick_host(url)
        if host in replaces:
            out.add(url.replace(host, replaces.get(host)))
    return out


def get_question_remove(url_set_in):
    u"""удаление вопроса с конца get """
    out = url_set_in.copy()
    for url in url_set_in:
        host, get = pick_host_get(url)
        if get == '/?':
            out.add(SCHEMA + host)
            out.remove(url)
    return out


def get_quote(get, safe="/"):
    u"""Превращает GET из кириллицы итд в %30%45 итд """
    if isinstance(get, unicode):
        return quote(get.encode('utf-8'), safe).decode('utf-8')
    return quote(get).decode('utf-8')


def mixed_quote_fix(host, get):
    u"""Иногда встречаются URL где часть уже quoted, а часть кириллица """
    escaped = re.compile('%[0-9A-F]{2}')
    safe_mixed = escaped.findall(get)
    if not safe_mixed:
        return
    safe = "#&(),*+:/;=?@~%"
    for char in set(escaped.findall(get)):
        safe = safe.replace(unquote(char), '')
    if isinstance(get, unicode):
        get = get.encode('utf-8')
    if isinstance(safe, unicode):
        safe = safe.encode('utf-8')
    new_get = quote(get, safe)
    return SCHEMA + host + new_get


def mixed_quote_add(out, host, get):
    u"""Добавление URL со смешанно quoted GET """
    mixed = mixed_quote_fix(host, get)
    if mixed:
        out.add(mixed)


def brackets_unquote(out, host, get):
    u"""Костыль, т.к. unquote не умеет в safe='[]' """
    out.add(SCHEMA + host + get.replace('%5B', '[').replace('%5D', ']'))


def curl_brakets_help(url_set_in):
    u"""добавляем вариант без {} """
    out = url_set_in.copy()
    for url in url_set_in:
        if url.find('{') > 0 and url.rfind('}') > 0:
            out.add(url.replace('{', '').replace('}', ''))
    return out


def quoted_cp1251(host, get):
    u"""Для URL которые были в quoted в cp1251, а не utf-8, raw """
    try:
        quoted_1251_get = str(get)
        decoded_1251_get = unquote(quoted_1251_get).decode('cp1251')
        return SCHEMA + host + decoded_1251_get
    except Exception:
        pass


QUOTE_USELESS = lambda url, quote: type(url) == type(quote) and url == quote
URL_QUOTE = lambda host, get, safe='/': SCHEMA + host + get_quote(get, safe)


def url_quote_main(out, url, host, get):
    u"""Все варианты экранирования """
    quoted = URL_QUOTE(host, get)
    if QUOTE_USELESS(url, quoted):
        return
    for safe in QUOTE_SAFE_VARIANTS:
        out.add(URL_QUOTE(host, get, safe))
    return True


URL_UNQUOTE = lambda host, get: SCHEMA + host + unquote(get)


def url_unquote_main(out, url, host, get):
    u"""Перевод из экранированного варианта в raw """
    if url.find('%') < 0:
        return
    unquoted = URL_UNQUOTE(host, get)
    if isinstance(url, unicode) and url == unquoted:
        return
    if ' ' not in unquoted:
        out.add(unquoted)
    return True


def url_cp1251_magic(out, host, get):
    u"""Для URL которые были в quoted в cp1251, а не utf-8, quoted """
    quoted1251 = quoted_cp1251(host, get)
    if quoted1251:
        if ' ' not in quoted1251:
            out.add(quoted1251)
        get_utf8 = pick_get(quoted1251).encode('utf-8')
        for safe in QUOTE_SAFE_VARIANTS:
            quote_safe = SCHEMA + host + quote(get_utf8, safe)
            if ' ' not in quote_safe:
                out.add(quote_safe)


def url_quote_unquote(url_set_in):
    u"""Вызов всего связанного с экранированием """
    out = url_set_in.copy()
    for url in url_set_in:
        host, get = pick_host_get(url)
        if not url_quote_main(out, url, host, get):
            continue
        if not url_unquote_main(out, url, host, get):
            continue
        brackets_unquote(out, host, get)
        url_cp1251_magic(out, host, get)
        mixed_quote_add(out, host, get)
    return out


def reduce_length(url_set_in):
    u"""Обрезание длинных URL """
    out = url_set_in.copy()
    for url in url_set_in:
        if len(url) > MAX_URL_LEN:
            out.add(__reduce_length(url))
            out.remove(url)
    return out


def __reduce_length(url):
    u"""Рекурсивно обрезает URL до ближайшего '/' вплоть до < 600 символов """
    if len(url) > MAX_URL_LEN:
        return __reduce_length(__strip_to_slash(url))
    return url


def __strip_to_slash(url, symbol='/'):
    u"""Обрезает URL с конца до первого нужного символа """
    return url[:url.rfind(symbol)]


def sharp_remove(url_set_in):
    u"""Разные браузеры по разному передают #, часть обрезает, часть нет """
    out = url_set_in.copy()
    for url in url_set_in:
        if pick_get(url).find('#') >= 0:
            out.add(url[:url.rfind('#')])
            out.add(url[:url.find('#')])
    return out


def valid_proto_only(url_set_in):
    u"""Избавляемся от левых строчек """
    is_valid_proto = lambda u: u.startswith(SCHEMA) and u.find('.') > 0
    return set([url for url in url_set_in if is_valid_proto(url)])


def is_only_url(url, proto_len=SCHEMA_LEN):
    u"""Домен целиком """
    get = pick_get(url, proto_len)
    return not get or get == '/'


def domain_only_urls(url_set_in):
    u"""Получаем список url-доменов (без GET) из множества """
    return set([pick_host(url) for url in url_set_in if is_only_url(url)])


def fulldomain_reduce(url_set_in):
    u"""Убираем те URL домен которых блочим целиком """
    out = set()
    domain_only = domain_only_urls(url_set_in)
    for url in url_set_in:
        host = pick_host(url)
        if host not in domain_only:
            out.add(url)
    for domain in domain_only:
        out.add(SCHEMA + domain)
    return out


def last_dot_in_get(url_set_in):
    u"""добавляем варианты с удалённой точкой с конца GET """
    out = set([re.sub(r'/?\.$', '', url, 1) for url in url_set_in])
    out |= url_set_in
    return out


def slash(url_set_in):
    """
    Поведение относительно вопросов и slash на конце варьируется
    Имеет смысл разбить на две функции, 1 ? 2 /
    """
    out = url_set_in.copy()
    for url in url_set_in:
        if url.endswith('/') and url.count('/') < 6:
            out.add(url.rstrip('/'))
            out.remove(url)
            continue
        if not url.endswith('/') and url.count('/') > 6:
            out.add(url + '/')
        if url.find('?') >= 0:
            out.add(url.replace('?', '/?', 1))
            if url.find('/?') >= 0:
                out.add(url.replace('/?', '?', 1))
            if url.endswith('?'):
                out.add(url.rstrip('?'))
    for url in out.copy():
        if url.find('//', SCHEMA_LEN) > 0:
            out.remove(url)
            out.add("%s%s" % (SCHEMA, url[SCHEMA_LEN:].replace('//', '/')))
    return out


def pick_host_get(url, proto_len=SCHEMA_LEN):
    u"""Чуть более эффективно, экономим вызов pick_host """
    delim = len(url)
    for char in '/?#':
        wdelim = url.find(char, proto_len)
        if wdelim >= 0:
            delim = min(delim, wdelim)
    return url[proto_len:delim], url[delim:]


def pick_host(url, proto_len=SCHEMA_LEN):
    u"""Получаем HOST """
    return pick_host_get(url, proto_len)[0]


def pick_get(url, proto_len=SCHEMA_LEN):
    """ Получаем GET """
    return pick_host_get(url, proto_len)[1]


FUNCTIONS_ORDERED = (
    valid_proto_only,
    lower,
    domain_change,
    # fulldomain_reduce,
    www,
    fqdn,
    idna,
    get_question_remove,
    slash,
    sharp_remove,
    cp1251,
    url_quote_unquote,
    reduce_length,
    last_dot_in_get,
    curl_brakets_help,
)
